
import javafx.application.Application;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.ClipboardContent;
import javafx.scene.input.DragEvent;
import javafx.scene.input.Dragboard;
import javafx.scene.input.MouseEvent;
import javafx.scene.input.TransferMode;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;
import jdk.nashorn.internal.codegen.types.Type;

public class DragNDropDemo extends Application {

    @Override
    public void start(Stage stage) {
        FlowPane root = new FlowPane(10.0, 10.0);
        root.setPadding(new Insets(10.0));
        configureDrop(root);

        // イメージの読み込み
        for (int index = 0; index < 6; index++) {
            Image image = new Image("resources/image" + index + ".jpg");
            ImageView view = new ImageView(image);
            view.setFocusTraversable(true);
            configureDrag(view);
            root.getChildren().add(view);
        }

        Scene scene = new Scene(root, 340, 250);

        stage.setTitle("Drag and Drop Demo");
        stage.setScene(scene);
        stage.show();
    }

    private void configureDrag(final ImageView view) {
        // ドラッグ開始
        view.setOnDragDetected(new EventHandler<MouseEvent>() {
            @Override
            public void handle(MouseEvent event) {
                // コピーと移動が使用できるドラッグボードを生成
                Dragboard dragboard
                        = view.startDragAndDrop(TransferMode.COPY_OR_MOVE);

                // ドラッグボードに保持させるコンテントを生成し、
                // ドラッグボードに保持させる
                ClipboardContent content = new ClipboardContent();
                content.putImage(view.getImage());
                dragboard.setContent(content);

                // ドラッグ中は半透明にし、ドラッグをしていることを分かるようにするy
                view.setOpacity(0.6);
            }
        });

        // ドラッグ終了
        view.setOnDragDone(new EventHandler<DragEvent>() {
            public void handle(DragEvent event) {
                if (event.isDropCompleted()) {
                    // ドロップが成功した場合
                    
                    TransferMode mode = event.getTransferMode();

                    if (mode == TransferMode.MOVE) {
                        // 移動の場合、コンテナから自分自身を削除する
                        Pane parent = (Pane) view.getParent();
                        parent.getChildren().remove(view);
                    } else if (mode == TransferMode.COPY) {
                        // コピーの場合、不透明に戻す
                        view.setOpacity(1.0);
                    }
                } else {
                    // ドラッグが失敗した場合、不透明に戻す
                    view.setOpacity(1.0);
                }
            }
        });
    }

    private void configureDrop(final FlowPane parent) {
        // ドロップ対象の上にマウスカーソルがある場合
        parent.setOnDragOver(new EventHandler<DragEvent>() {
            public void handle(DragEvent event) {
                // ドラッグボードを取得
                Dragboard dragboard = event.getDragboard();
                if (dragboard.hasImage()) {
                    // ドラッグされているのがイメージである場合
                    // コピーもしくは移動で受け入れ可能であることを示す
                    event.acceptTransferModes(TransferMode.COPY_OR_MOVE);
                }
            }
        });

        // ドロップ時
        parent.setOnDragDropped(new EventHandler<DragEvent>() {
            @Override
            public void handle(DragEvent event) {
                Dragboard dragboard = event.getDragboard();

                if (dragboard.hasImage()) {
                    // ドラッグされているものがイメージであれば、コンテナに追加する
                    parent.getChildren().add(new ImageView(dragboard.getImage()));

                    // ドラッグ成功を返す
                    event.setDropCompleted(true);
                } else {
                    // ドラッグ失敗を返す
                    event.setDropCompleted(false);
                }
            }
        });
    }

    public static void main(String... args) {
        launch(args);
    }
}
